@@ -11,14 +11,17 @@ mod macros;
 
 mod add;
 mod add_mod;
-mod and;
+mod bit_and;
+mod bit_not;
+mod bit_or;
+mod bit_xor;
+mod bits;
 mod cmp;
 mod div;
 mod encoding;
 mod from;
 mod mul;
 mod neg_mod;
-mod or;
 mod shl;
 mod shr;
 mod sqrt;
@@ -28,11 +31,10 @@ mod sub_mod;
 #[cfg(feature = "generic-array")]
 mod array;
 
-mod bits;
-#[cfg(feature = "rand")]
+#[cfg(feature = "rand_core")]
 mod rand;
 
-use crate::{Concat, Encoding, Integer, Limb, Split};
+use crate::{Concat, Encoding, Integer, Limb, Split, Zero};
 use core::fmt;
 use subtle::{Choice, ConditionallySelectable};
 
@@ -124,19 +126,22 @@ impl<const LIMBS: usize> Default for UInt<LIMBS> {
     }
 }
 
-impl<const LIMBS: usize> Integer for UInt<LIMBS>
-where
-    Self: Encoding,
-{
-    const ZERO: Self = Self::ZERO;
+impl<const LIMBS: usize> Integer for UInt<LIMBS> {
     const ONE: Self = Self::ONE;
     const MAX: Self = Self::MAX;
 
     fn is_odd(&self) -> Choice {
-        self.is_odd()
+        self.limbs
+            .first()
+            .map(|limb| limb.is_odd())
+            .unwrap_or_else(|| Choice::from(0))
     }
 }
 
+impl<const LIMBS: usize> Zero for UInt<LIMBS> {
+    const ZERO: Self = Self::ZERO;
+}
+
 impl<const LIMBS: usize> fmt::Display for UInt<LIMBS> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::UpperHex::fmt(self, f)
